/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 *
 * hal_LCD.c
 *
 * Hardware abstraction layer for the FH-1138P Segmented LCD
 *
 * September 2014
 * E. Chen
 *
 ******************************************************************************/
#include <stdbool.h>
#include "hal_LCD.h"
#include "driverlib/MSP430FR2xx_4xx/driverlib.h"
#include "string.h"
#include "utilitaires.h"

// LCD memory map for numeric digits P3.1 P3.0 P3.3 P3.2
const char digit[10][2] =
{
    {0xFC, 0x28},  /* "0" LCD segments a+b+c+d+e+f+k+q */
    {0x0C, 0x08},  /* "1" */
    {0xDB, 0x00},  /* "2" */
    {0x9F, 0x00},  /* "3" */
    {0x2F, 0x00},  /* "4" */
    {0xB7, 0x00},  /* "5" */
    {0xF7, 0x00},  /* "6" */
    {0x3C, 0x00},  /* "7" */
    {0xFF, 0x00},  /* "8" */
    {0xBF, 0x00}   /* "9" */
};

// LCD memory map for uppercase letters
const char alphabetBig[26][2] =
{
    {0x7F, 0x00},  /* "A" LCD segments b+c+d+e+f+g+m */
    {0x9E, 0x50},  /* "B" */
    {0xF0, 0x00},  /* "C" */
    {0x9C, 0x50},  /* "D" */
    {0xF3, 0x00},  /* "E" */
    {0x73, 0x00},  /* "F" */
    {0xF6, 0x00},  /* "G" */
    {0x6F, 0x00},  /* "H" */
    {0x90, 0x50},  /* "I" */
    {0xCC, 0x00},  /* "J" */
    {0x61, 0x88},  /* "K" */
    {0xE0, 0x00},  /* "L" */
    {0x6C, 0x0A},  /* "M" */
    {0x6C, 0x82},  /* "N" */
    {0xFC, 0x00},  /* "O" */
    {0x7B, 0x00},  /* "P" */
    {0xFC, 0x80},  /* "Q" */
    {0x7B, 0x80},  /* "R" */
    {0xB7, 0x00},  /* "S" */
    {0x10, 0x50},  /* "T" */
    {0xEC, 0x00},  /* "U" */
    {0x60, 0x28},  /* "V" */
    {0x6C, 0xA0},  /* "W" */
    {0x00, 0xAA},  /* "X" */
    {0x00, 0x4A},  /* "Y" */
    {0x90, 0x28}   /* "Z" */
};

void Init_LCD()
{
    LCD_Clear();
    // L0~L26 & L36~L39 pins selected
    LCD_E_setPinAsLCDFunctionEx(LCD_E_BASE, LCD_E_SEGMENT_LINE_0, LCD_E_SEGMENT_LINE_26);
    LCD_E_setPinAsLCDFunctionEx(LCD_E_BASE, LCD_E_SEGMENT_LINE_36, LCD_E_SEGMENT_LINE_39);

    LCD_E_initParam initParams = LCD_E_INIT_PARAM;
    initParams.clockDivider = LCD_E_CLOCKDIVIDER_3;
    initParams.muxRate = LCD_E_4_MUX;
    initParams.segments = LCD_E_SEGMENTS_ENABLED;

    // Init LCD as 4-mux mode
    LCD_E_init(LCD_E_BASE, &initParams);

    // LCD Operation - Mode 3, internal 3.02v, charge pump 256Hz
    LCD_E_setVLCDSource(LCD_E_BASE, LCD_E_INTERNAL_REFERENCE_VOLTAGE, LCD_E_EXTERNAL_SUPPLY_VOLTAGE);
    LCD_E_setVLCDVoltage(LCD_E_BASE, LCD_E_REFERENCE_VOLTAGE_2_96V);

    LCD_E_enableChargePump(LCD_E_BASE);
    LCD_E_setChargePumpFreq(LCD_E_BASE, LCD_E_CHARGEPUMP_FREQ_16);

    // Clear LCD memory
    LCD_E_clearAllMemory(LCD_E_BASE);

    // Configure COMs and SEGs
    // L0 = COM0, L1 = COM1, L2 = COM2, L3 = COM3
    LCD_E_setPinAsCOM(LCD_E_BASE, LCD_E_SEGMENT_LINE_0, LCD_E_MEMORY_COM0);
    LCD_E_setPinAsCOM(LCD_E_BASE, LCD_E_SEGMENT_LINE_1, LCD_E_MEMORY_COM1);
    LCD_E_setPinAsCOM(LCD_E_BASE, LCD_E_SEGMENT_LINE_2, LCD_E_MEMORY_COM2);
    LCD_E_setPinAsCOM(LCD_E_BASE, LCD_E_SEGMENT_LINE_3, LCD_E_MEMORY_COM3);

    // Select to display main LCD memory
    LCD_E_selectDisplayMemory(LCD_E_BASE, LCD_E_DISPLAYSOURCE_MEMORY);

    // Turn on LCD
    LCD_E_on(LCD_E_BASE);
}

/*
 * Scrolls input string across LCD screen from left to right
 */
void displayScrollText(const char *msg)
{
    int length = strlen(msg);
    uint16_t i;
    int16_t s = 5;
    int16_t j;
    char buffer[6] = "      ";
    for (i=0; i<length+7; i++) // faire pour toute la longueur du message plus 7 caractres
    {
        for (j=0; j<6; j++){buffer[j] = ' ';} // Mettre  expace le buffer d'affichage
        for (j=0; j<length; j++) // remplir le buffer d'affichage
        {
            if (((s+j) >= 0) && ((s+j) < 6))
                buffer[s+j] = msg[j];
        }
        s--;

        showChar(buffer[0], 6);
        showChar(buffer[1], 5);
        showChar(buffer[2], 4);
        showChar(buffer[3], 3);
        showChar(buffer[4], 2);
        showChar(buffer[5], 1);

        SleepMS(250);
    }
}

/*
 * Displays input character at given LCD digit/position
 * Only spaces, '+', '-', '}' (->), ',', '.', numeric digits, and uppercase letters are accepted characters
 */
void showChar(char c, unsigned int position)
{
    unsigned int msq;
    msq = 0x0000;
    switch (position)
    {
        case 1 : position = 2; break;
        case 2 : position = 3; break;
        case 3 : position = 4; msq = 0x0400; break;
        case 4 : position = 5; break;
        case 5 : position = 1; break;
        case 6 : position = 9; msq = 0x0500; break;
        default : position = 2; break;
    }
    if ((c == ' ') || (c == '='))
    {
        // Display space
        LCDMEMW[position] = LCDMEMW[position] & msq;
    }
    else if (c == '\x0')
    {
        //  rien si caractre nul
    }
    else if ((c >= '0') && (c <= '9'))
    {
        // Display digit
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (digit[c-48][0] | (digit[c-48][1] << 8));
    }
    else if ((c >= 'A') && (c <= 'Z'))
    {
        // Display alphabet
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (alphabetBig[c-65][0] | (alphabetBig[c-65][1] << 8));
    }
    else if (c == '-')
    {
        // Display alphabet
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (0x0003);
    }
    else if ((c == ',') || (c == '.'))
    {
        // Display alphabet
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (0x2000);
    }
    else if (c == '+')
    {
        // Display alphabet
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (0x5003);
    }
    else if (c == '}') // ->
    {
        // Display alphabet
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (0x2201);
    }
    else if (c == '*') // *
    {
        // Display alphabet
        LCDMEMW[position] = (LCDMEMW[position] & msq) | (0xAA03);
    }
    else
    {
        // Turn all segments on if character is not a space, digit, or uppercase letter
        LCDMEMW[position] = 0xFFFF;
    }
}

/*
 * Efface tous les digits
 */
void LCD_Clear()
{
    LCDMEMW[2] = 0;
    LCDMEMW[3] = 0;
    LCDMEMW[4] = 0;
    LCDMEMW[5] = 0;
    LCDMEMW[1] = 0;
    LCDMEMW[9] = 0;
    LCDMEM[12] = 0;
    LCDMEM[13] = 0;
}

/*
 * Efface les digits chiffres/textes
 */
void LCD_ClearText()
{
    int i;
    for (i=6; i > 0; i--) {showChar(' ',i);}
}


// Affiche le point en position Pos
// 0 efface tous les points
void LCD_displayDot(int pos)
{
    if (pos > 5) {pos = 5;}
    LCDMEM[11] = LCDMEM[11] & 0xFE;
    LCDMEM[9]  = LCDMEM[9] & 0xFE;
    LCDMEM[7]  = LCDMEM[7] & 0xFE;
    LCDMEM[5]  = LCDMEM[5] & 0xFE;
    LCDMEM[3]  = LCDMEM[3] & 0xFE;
    switch (pos)
    {
        case 0 : break;
        case 1 : LCDMEM[5] = LCDMEM[5] | 0x01; break;
        case 2 : LCDMEM[7] = LCDMEM[7] | 0x01; break;
        case 3 : LCDMEM[9] = LCDMEM[9] | 0x01; break;
        case 4 : LCDMEM[11] = LCDMEM[11] | 0x01; break;
        case 5 : LCDMEM[3] = LCDMEM[3] | 0x01; break;
    }
}



//
// Ecrit un nombre cadr  droite de l'afficheur
// En entre : number = nombre  crire
//
void LCD_writeIntRight (uint32_t number)  // A cool function that moves number right to left
{
   int i;
   if (number > 999999) { // si dpassement
       for (i = 6; i > 0; i--) {showChar('*',i);}      // On rempli avec des toiles
   } else {
       i = 1;
       do      // Extract each digit in number, put in an integer array, and count total length also
       {
          showChar((number % 10) | 0x30,i);
          i = i+1;
          number = number / 10;     // remove the least significant character from number
       } while (number > 0);
   }
}

//
// Ecrit un texte cadr  gauche de l'afficheur
// msg = texte  afficher
// lg = longueur du texte (tronqu  la taille max de l'afficheur par la fonction
//
void LCD_writeTxtLeft (const char *buf, unsigned int lg)
{
    int i;
    if (lg > 6) {lg = 6;}
    for (i = 0; i < lg; i++){showChar(buf[i], 6-i);}
}


// Affiche (True) / efface (False) Antenne en fonction de OnOff
void LCD_displayAnt(bool OnOff)
{
    if (OnOff) {LCDMEM[9] = LCDMEM[9] | 0x04;} else {LCDMEM[9] = LCDMEM[9] & 0xFB;}
}


// Affiche (True) / efface (False) Timer en fonction de OnOff
void LCD_displayTmr(bool OnOff)
{
    if (OnOff) {LCDMEM[12] = LCDMEM[12] | 0x08;} else {LCDMEM[12] = LCDMEM[12] & 0xF7;}
}

// Affiche (True) / efface (False) ! en fonction de OnOff
void LCD_displayAlarm(bool OnOff)
{
    if (OnOff) {LCDMEM[12] = LCDMEM[12] | 0x01;} else {LCDMEM[12] = LCDMEM[12] & 0xFE;}
}

// Affiche (True) / efface (False) Coeur en fonction de OnOff
void LCD_displayHeart(bool OnOff)
{
    if (OnOff) {LCDMEM[12] = LCDMEM[12] | 0x04;} else {LCDMEM[12] = LCDMEM[12] & 0xFB;}
}

// Affiche (OnOff True) / efface (OnOff False) Niveau de batterie. Level = 0  6
void LCD_displayLevel(unsigned int level, bool OnOff)
{
    if (OnOff) {
        LCDMEM[13] = 0x01; // affiche les []
        LCDMEM[12] = (LCDMEM[12] & 0x0F);
        if (level > 6) {level = 6;}
        switch (level)
        {
            case 6 : LCDMEM[13] = LCDMEM[13] | 0x02;
            case 5 : LCDMEM[12] = LCDMEM[12] | 0x20;
            case 4 : LCDMEM[13] = LCDMEM[13] | 0x04;
            case 3 : LCDMEM[12] = LCDMEM[12] | 0x40;
            case 2 : LCDMEM[13] = LCDMEM[13] | 0x08;
            case 1 : LCDMEM[12] = LCDMEM[12] | 0x80;
        }
    } else {
        LCDMEM[13] = 0x00; // efface tout
        LCDMEM[12] = (LCDMEM[12] & 0x0F);
    }
}




